package de.pdbm;

import java.lang.foreign.Arena;
import java.lang.foreign.FunctionDescriptor;
import java.lang.foreign.Linker;
import java.lang.foreign.MemorySegment;

import static java.lang.foreign.ValueLayout.ADDRESS;
import static java.lang.foreign.ValueLayout.JAVA_LONG;

import java.lang.invoke.MethodHandle;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

/**
 * 
 * @author bernd
 *
 */
public class StdLibTest {

	private static MethodHandle strlen; // C's strlen() function
	private static MethodHandle strcat; // C's strcat() function

	@BeforeAll
	public static void locateFunctions() {
		Linker linker = Linker.nativeLinker();
		strlen = linker.downcallHandle(linker.defaultLookup().find("strlen").get(),
				FunctionDescriptor.of(JAVA_LONG, ADDRESS));
		strcat = linker.downcallHandle(linker.defaultLookup().find("strcat").get(),
				FunctionDescriptor.of(ADDRESS, ADDRESS, ADDRESS));
	}


	@Test
	public void strlen() throws Throwable {
		try (Arena arena = Arena.openConfined()) {
			MemorySegment segment = arena.allocateUtf8String("Hello");
			int length = (int) (long) strlen.invoke(segment);
			Assertions.assertSame(5, length);
		}
	}

	
	@Test
	public void foo() throws Throwable {
		String result = strcat("hello", "world");
	}
	
	/**
	 * This is a one to one copy of strcat test taken from  StdLibTest.java test from OpenJDK
	 */
    String strcat(String s1, String s2) throws Throwable {
        try (var arena = Arena.openConfined()) {
            MemorySegment buf = arena.allocate(s1.length() + s2.length() + 1);
            buf.setUtf8String(0, s1);
            MemorySegment other = arena.allocateUtf8String(s2);
            return ((MemorySegment)strcat.invokeExact(buf, other)).getUtf8String(0);
        }
    }
	
    
}
